// Treiber fr TPIC8101 Klopfsensorinterface
//
// Programmierer:		Peter Diener
// 2010



// TPIC Steuerpins
#define PORT_TPIC_test			PORTC
#define PORT_TPIC_Int_Hold	PORTC
#define PORT_TPIC_CS				PORTB
#define DDR_TPIC_test				DDRC
#define DDR_TPIC_Int_Hold		DDRC
#define DDR_TPIC_CS					DDRB

#define Pin_TPIC_test				1
#define Pin_TPIC_Int_Hold		2
#define Pin_TPIC_CS					2


// Prescaler fr Verwendung eines 10 MHz Quarzes ausgelegt.
#define TPIC_prescaler_SDO	0b01001000

typedef enum {CH1 = 0, CH2 = 1} TPIC_ch;


// Momentanwerte
extern unsigned char TPIC_current_channel;
extern unsigned char TPIC_current_freq;
extern unsigned char TPIC_current_gain;
extern unsigned char TPIC_current_integration_time_constant;


// Verfgbare Funktionen fr den Anwender:

// Analogen Eingangskanal auswhlen.
// Beispiel: TPIC_select_channel(CH1);
unsigned char TPIC_select_channel (TPIC_ch channel);

// Frequenz des Analogbandpasses einstellen.
// Werte knnen von 0 bis 63 eingestellt werden.
void TPIC_set_bandpass_frequency (unsigned char frequency);

// Analoge Verstrkung einstellen.
// Werte knnen von 0 bis 63 eingestellt werden.
void TPIC_set_gain(unsigned char gain);

// Integrationszeitkonstante einstellen
// Werte knnen von 0 bis 31 eingestellt werden.
void TPIC_set_integration_time_constant(unsigned char time);

// Takt Vorteiler und SDO Funktion auswhlen.
// Diese Funktion muss einmal am Anfang aufgerufen werden.
void TPIC_init(void);

// Klopflautstrke auslesen und Integrator rcksetzen
unsigned int TPIC_get_knock_value(void);

// Ende der verfgbaren Funktionen


// Frequenztabelle fr TPIC Bandpassfilter
// Bitte auf alle 64 einstellbaren Frequenzen erweitern!
#define		TPIC_freq_1_22_kHz	0 
#define		TPIC_freq_1_26_kHz	1 
#define		TPIC_freq_1_31_kHz	2 
#define		TPIC_freq_1_35_kHz	3 
#define		TPIC_freq_1_40_kHz	4 
#define		TPIC_freq_1_45_kHz	5 
#define		TPIC_freq_1_51_kHz	6 
#define		TPIC_freq_1_57_kHz	7 
#define		TPIC_freq_1_63_kHz	8 
#define		TPIC_freq_1_71_kHz	9 
#define		TPIC_freq_1_78_kHz	10
#define		TPIC_freq_1_87_kHz	11
#define		TPIC_freq_1_96_kHz	12
#define		TPIC_freq_2_07_kHz	13
#define		TPIC_freq_2_18_kHz	14
#define		TPIC_freq_2_31_kHz	15
#define		TPIC_freq_2_46_kHz	16
#define		TPIC_freq_2_54_kHz	17
#define		TPIC_freq_2_62_kHz	18
#define		TPIC_freq_2_71_kHz	19
#define		TPIC_freq_2_81_kHz	20
#define		TPIC_freq_2_92_kHz	21
#define		TPIC_freq_3_03_kHz	22
#define		TPIC_freq_3_15_kHz	23
#define		TPIC_freq_3_28_kHz	24
#define		TPIC_freq_3_43_kHz	25
#define		TPIC_freq_3_59_kHz	26
#define		TPIC_freq_3_76_kHz	27
#define		TPIC_freq_3_95_kHz	28
#define		TPIC_freq_4_16_kHz	29
#define		TPIC_freq_4_39_kHz	30
#define		TPIC_freq_4_66_kHz	31
#define		TPIC_freq_4_95_kHz	32
#define		TPIC_freq_5_12_kHz	33
#define		TPIC_freq_5_29_kHz	34
#define		TPIC_freq_5_48_kHz	35
#define		TPIC_freq_5_68_kHz	36
#define		TPIC_freq_5_90_kHz	37
#define		TPIC_freq_6_12_kHz	38
#define		TPIC_freq_6_37_kHz	39
#define		TPIC_freq_6_64_kHz	40
#define		TPIC_freq_6_94_kHz	41
#define		TPIC_freq_7_27_kHz	42
#define		TPIC_freq_7_63_kHz	43
#define		TPIC_freq_8_02_kHz	44
#define		TPIC_freq_8_46_kHz	45
#define		TPIC_freq_8_95_kHz	46
#define		TPIC_freq_9_50_kHz	47
#define		TPIC_freq_10_12_kHz	48
#define		TPIC_freq_10_46_kHz	49
#define		TPIC_freq_10_83_kHz	50
#define		TPIC_freq_11_22_kHz	51
#define		TPIC_freq_11_65_kHz	52
#define		TPIC_freq_12_10_kHz	53
#define		TPIC_freq_12_60_kHz	54
#define		TPIC_freq_13_14_kHz	55
#define		TPIC_freq_13_72_kHz	56
#define		TPIC_freq_14_36_kHz	57
#define		TPIC_freq_15_07_kHz	58
#define		TPIC_freq_15_84_kHz	59
#define		TPIC_freq_16_71_kHz	60
#define		TPIC_freq_17_67_kHz	61
#define		TPIC_freq_18_76_kHz	62
#define		TPIC_freq_19_98_kHz	63


// Tabelle Fr analoge Verstrkungswerte
#define		TPIC_gain_2_000 	0 
#define		TPIC_gain_1_882		1 
#define		TPIC_gain_1_778		2 
#define		TPIC_gain_1_684		3 
#define		TPIC_gain_1_600		4 
#define		TPIC_gain_1_523		5 
#define		TPIC_gain_1_455		6 
#define		TPIC_gain_1_391		7 
#define		TPIC_gain_1_333		8 
#define		TPIC_gain_1_280		9 
#define		TPIC_gain_1_231		10
#define		TPIC_gain_1_185		11
#define		TPIC_gain_1_143		12
#define		TPIC_gain_1_063		13
#define		TPIC_gain_1_000  	14
#define		TPIC_gain_0_944		15
#define		TPIC_gain_0_895		16
#define		TPIC_gain_0_850 	17
#define		TPIC_gain_0_810 	18
#define		TPIC_gain_0_773		19
#define		TPIC_gain_0_739		20
#define		TPIC_gain_0_708		21
#define		TPIC_gain_0_680 	22
#define		TPIC_gain_0_654		23
#define		TPIC_gain_0_630 	24
#define		TPIC_gain_0_607		25
#define		TPIC_gain_0_586		26
#define		TPIC_gain_0_567		27
#define		TPIC_gain_0_548		28
#define		TPIC_gain_0_500  	29
#define		TPIC_gain_0_471		30
#define		TPIC_gain_0_444		31
#define		TPIC_gain_0_421		32
#define		TPIC_gain_0_400  	33
#define		TPIC_gain_0_381		34
#define		TPIC_gain_0_364		35
#define		TPIC_gain_0_348		36
#define		TPIC_gain_0_333		37
#define		TPIC_gain_0_320 	38
#define		TPIC_gain_0_308		39
#define		TPIC_gain_0_296		40
#define		TPIC_gain_0_286		41
#define		TPIC_gain_0_276		42
#define		TPIC_gain_0_267		43
#define		TPIC_gain_0_258		44
#define		TPIC_gain_0_250 	45
#define		TPIC_gain_0_236		46
#define		TPIC_gain_0_222		47
#define		TPIC_gain_0_211		48
#define		TPIC_gain_0_200  	49
#define		TPIC_gain_0_190 	50
#define		TPIC_gain_0_182		51
#define		TPIC_gain_0_174		52
#define		TPIC_gain_0_167		53
#define		TPIC_gain_0_160 	54
#define		TPIC_gain_0_154		55
#define		TPIC_gain_0_148		56
#define		TPIC_gain_0_143		57
#define		TPIC_gain_0_138		58
#define		TPIC_gain_0_133		59
#define		TPIC_gain_0_129		60
#define		TPIC_gain_0_125		61
#define		TPIC_gain_0_118		62
#define		TPIC_gain_0_111		63

// Zeittabelle fr Integrationszeitkonstanten
#define 	TPIC_intTime_40_us 	0 
#define 	TPIC_intTime_45_us 	1 
#define 	TPIC_intTime_50_us 	2 
#define 	TPIC_intTime_55_us 	3 
#define 	TPIC_intTime_60_us 	4 
#define 	TPIC_intTime_65_us 	5 
#define 	TPIC_intTime_70_us 	6 
#define 	TPIC_intTime_75_us 	7 
#define 	TPIC_intTime_80_us 	8 
#define 	TPIC_intTime_90_us 	9 
#define 	TPIC_intTime_100_us	10
#define 	TPIC_intTime_110_us	11
#define 	TPIC_intTime_120_us	12
#define 	TPIC_intTime_130_us	13
#define 	TPIC_intTime_140_us	14
#define 	TPIC_intTime_150_us	15
#define 	TPIC_intTime_160_us	16
#define 	TPIC_intTime_180_us	17
#define 	TPIC_intTime_200_us	18
#define 	TPIC_intTime_220_us	19
#define 	TPIC_intTime_240_us	20
#define 	TPIC_intTime_260_us	21
#define 	TPIC_intTime_280_us	22
#define 	TPIC_intTime_300_us	23
#define 	TPIC_intTime_320_us	24
#define 	TPIC_intTime_360_us	25
#define 	TPIC_intTime_400_us	26
#define 	TPIC_intTime_440_us	27
#define 	TPIC_intTime_480_us	28
#define 	TPIC_intTime_520_us	29
#define 	TPIC_intTime_560_us	30
#define 	TPIC_intTime_600_us	31
